/*****************************************************************************

        FilterResize.h
        Author: Laurent de Soras, 2011

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if ! defined (FilterResize_HEADER_INCLUDED)
#define	FilterResize_HEADER_INCLUDED

#if defined (_MSC_VER)
	#pragma once
	#pragma warning (4 : 4250)
#endif



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"conc/ObjPool.h"
#include	"AllocAlign.h"
#include	"avstp.h"
#include	"AvstpWrapper.h"
#include	"ResizeData.h"
#include	"ResizeDataFactory.h"
#include	"Scaler.h"
#include	"types.h"

#include	<emmintrin.h>

#include	<vector>
#include	<memory>



class ContFirInterface;

class FilterResize
{

/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

public:

	enum Dir
	{
		Dir_H = 0,
		Dir_V,

		Dir_NBR_ELT
	};

	typedef	FilterResize	ThisType;

	explicit			FilterResize (int src_width, int src_height, int dst_width, int dst_height, double win_x, double win_y, double win_w, double win_h, ContFirInterface &kernel_fnc_h, ContFirInterface &kernel_fnc_v, double kernel_scale_h, double kernel_scale_v, bool norm_flag, double norm_val_h, double norm_val_v, double center_pos_h, double center_pos_v, bool sse2_flag);
	virtual			~FilterResize () {}

	void				process_plane (uint8_t *dst_msb_ptr, uint8_t *dst_lsb_ptr, const uint8_t *src_msb_ptr, const uint8_t *src_lsb_ptr, int stride_dst, int stride_src);



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

protected:



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

	enum PassType
	{
		PassType_NONE = 0,
		PassType_RESIZE,
		PassType_TRANSPOSE,

		PassType_NBR_ELT
	};

	enum {			MAX_NBR_PASSES	= 4	};	// 2 * (transpose + resize)
	enum {			MAX_BUF_SIZE	= 65536	};	// Number of pixels (float)

	class TaskRszGlobal
	{
	public:
		FilterResize *	_this_ptr;
		uint8_t *	_dst_msb_ptr;
		uint8_t *	_dst_lsb_ptr;
		const uint8_t *
						_src_msb_ptr;
		const uint8_t *
						_src_lsb_ptr;
		int			_stride_dst;
		int			_stride_src;
		int			_offset_crop;
	};

	class TaskRsz
	{
	public:
		const TaskRszGlobal *
							_glob_data_ptr;
		int				_dst_beg [Dir_NBR_ELT];
		int				_work_dst [Dir_NBR_ELT];
		int				_src_beg [Dir_NBR_ELT];
		int				_src_end [Dir_NBR_ELT];
	};

	typedef	conc::LockFreeCell <TaskRsz>	TaskRszCell;

	void				process_plane_bypass (uint8_t *dst_msb_ptr, uint8_t *dst_lsb_ptr, const uint8_t *src_msb_ptr, const uint8_t *src_lsb_ptr, int stride_dst, int stride_src);
	void				process_plane_normal (uint8_t *dst_msb_ptr, uint8_t *dst_lsb_ptr, const uint8_t *src_msb_ptr, const uint8_t *src_lsb_ptr, int stride_dst, int stride_src);
	void				process_tile (TaskRszCell &tr_cell);

	void				conv_lsb_msb_to_float_cpp (float *dst_ptr, const uint8_t *src_msb_ptr, const uint8_t *src_lsb_ptr, int w, int h, int stride_dst, int stride_src);
	void				conv_lsb_msb_to_float_sse (float *dst_ptr, const uint8_t *src_msb_ptr, const uint8_t *src_lsb_ptr, int w, int h, int stride_dst, int stride_src);

	void				conv_float_to_lsb_msb_cpp (uint8_t *dst_msb_ptr, uint8_t *dst_lsb_ptr, const float *src_ptr, int w, int h, int stride_dst, int stride_src);
	void				conv_float_to_lsb_msb_sse (uint8_t *dst_msb_ptr, uint8_t *dst_lsb_ptr, const float *src_ptr, int w, int h, int stride_dst, int stride_src);

	void				transpose (float *dst_ptr, const float *src_ptr, int w, int h, int stride_dst, int stride_src);
	bool				is_kernel_neutral (Dir di) const;

	inline bool		has_buf_src (int pass) const;
	inline bool		has_buf_dst (int pass) const;
	void				compute_req_src_tile_size (int &tw, int &th, int dw, int dh) const;

	static void		redirect_task_resize (avstp_TaskDispatcher *dispatcher_ptr, void *data_ptr);

	AvstpWrapper &	_avstp;
	conc::CellPool <TaskRsz>
						_task_rsz_pool;

	int				_src_size [Dir_NBR_ELT];
	int				_dst_size [Dir_NBR_ELT];
	double			_win_pos [Dir_NBR_ELT];
	double			_win_size [Dir_NBR_ELT];
	double			_kernel_scale [Dir_NBR_ELT];
	bool				_kernel_force_flag [Dir_NBR_ELT];
	ContFirInterface *
						_kernel_ptr_arr [Dir_NBR_ELT];
	bool				_norm_flag;
	double			_norm_val [Dir_NBR_ELT];
	double			_center_pos [Dir_NBR_ELT];
	bool				_sse2_flag;

	conc::ObjPool <ResizeData>
						_pool;
	std::auto_ptr <ResizeDataFactory>
						_factory_aptr;

	int				_crop_pos [Dir_NBR_ELT];
	int				_crop_size [Dir_NBR_ELT];

	std::auto_ptr <Scaler>				// 0 if bypassed
						_scaler_aptr [Dir_NBR_ELT];

	bool				_resize_flag [Dir_NBR_ELT];
	PassType			_roadmap [MAX_NBR_PASSES];
	int				_tile_size_dst [Dir_NBR_ELT];
	int				_nbr_passes;		// 0 = bypass
	bool				_buffer_flag;

	void (ThisType::*
						_conv_lsb_msb_to_float_ptr) (float *dst_ptr, const uint8_t *src_msb_ptr, const uint8_t *src_lsb_ptr, int w, int h, int stride_dst, int stride_src);
	void (ThisType::*
						_conv_float_to_lsb_msb_ptr) (uint8_t *dst_msb_ptr, uint8_t *dst_lsb_ptr, const float *src_ptr, int w, int h, int stride_dst, int stride_src);



/*\\\ FORBIDDEN MEMBER FUNCTIONS \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

private:

						FilterResize ();
						FilterResize (const FilterResize &other);
	FilterResize &	operator = (const FilterResize &other);
	bool				operator == (const FilterResize &other) const;
	bool				operator != (const FilterResize &other) const;

};	// class FilterResize



//#include	"FilterResize.hpp"



#endif	// FilterResize_HEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
