/*****************************************************************************

        Mutex.hpp
        Author: Laurent de Soras, 2011

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if ! defined (conc_Mutex_CODEHEADER_INCLUDED)
#define	conc_Mutex_CODEHEADER_INCLUDED



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#if ! defined (_MSC_VER)
	#include <stdexcept>

	#include	<cassert>
#endif



namespace conc
{



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



Mutex::Mutex ()
:	_crit_sec ()
{
#if defined (_MSC_VER)

	::InitializeCriticalSection (&_crit_sec);

#else

	int            err = pthread_mutex_init (&_crit_sec, 0);
	if (err != 0)
	{
		assert (false);
		throw std::runtime_error ("conc::Mutex::Mutex(): Cannot create mutex.");
	}

#endif
}



Mutex::~Mutex ()
{
#if defined (_MSC_VER)

	::DeleteCriticalSection (&_crit_sec);

#else

	const int      err = ::pthread_mutex_destroy (&_crit_sec);
	assert (err == 0);

#endif
}



void	Mutex::lock ()
{
#if defined (_MSC_VER)

	::EnterCriticalSection (&_crit_sec);

#else

	const int      err = ::pthread_mutex_lock (&_crit_sec);
	if (err != 0)
	{
		assert (false);
		throw std::runtime_error ("conc::Mutex::lock(): Cannot lock mutex.");
	}

#endif
}



void	Mutex::unlock ()
{
#if defined (_MSC_VER)

	::LeaveCriticalSection (&_crit_sec);

#else

	int            err = ::pthread_mutex_unlock (&_crit_sec);
	if (err != 0)
	{
		assert (false);
		throw std::runtime_error ("conc::Mutex::lock(): Cannot unlock mutex.");
	}

#endif
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



}	// namespace conc



#endif	// conc_Mutex_CODEHEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
