/*****************************************************************************

        fnc.hpp
        Author: Laurent de Soras, 2010

--- Legal stuff ---

This program is free software. It comes without any warranty, to
the extent permitted by applicable law. You can redistribute it
and/or modify it under the terms of the Do What The Fuck You Want
To Public License, Version 2, as published by Sam Hocevar. See
http://sam.zoy.org/wtfpl/COPYING for more details.

*Tab=3***********************************************************************/



#if ! defined (fnc_CODEHEADER_INCLUDED)
#define	fnc_CODEHEADER_INCLUDED



/*\\\ INCLUDE FILES \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PUBLIC \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include	"def.h"
#include	"types.h"

#include	<cassert>
#include	<cmath>

#if defined (_MSC_VER)
	#include <intrin.h>
	#pragma intrinsic (_BitScanReverse)
#endif



template <class T>
T	limit (T x, T mi, T ma)
{
	return ((x < mi) ? mi : ((x > ma) ? ma : x));
}



template <class T>
bool	is_pow_2 (T x)
{
	return ((x & -x) == x);
}



double	round (double x)
{
	return (floor (x + 0.5));
}



int	round_int (double x)
{
	return (int (floor (x + 0.5)));
}



int	floor_int (double x)
{
	return (int (floor (x)));
}



int	ceil_int (double x)
{
	return (int (ceil (x)));
}



int	conv_int_fast (float x)
{
	int				p;

#if defined (_MSC_VER)

 #if defined (_WIN64) || defined (_M_X64)

	p = conv_int_fast (double (x));

 #else

	__asm
	{
		fld				x;
		fistp				p;
	}

 #endif

#elif defined (__GNUC__)

	asm (
		"flds				%[x]		\n"
		"fistpl			%[v]		\n"
	:	[v]	"=m"	(p)
	:	[x]	"m"	(x)
	:	
	);

#else

	p = int (x);

#endif

	return (p);
}



int	conv_int_fast (double x)
{
	int				p;

#if defined (_MSC_VER)

 #if defined (_WIN64) || defined (_M_X64)

	const __m128d	x_128d = _mm_set_sd (x);
	p = _mm_cvtsd_si32 (x_128d);

 #else

	__asm
	{
		fld				x;
		fistp				p;
	}

 #endif

#elif defined (__GNUC__)

	asm (
		"flds				%[x]		\n"
		"fistpl			%[v]		\n"
	:	[v]	"=m"	(p)
	:	[x]	"m"	(x)
	:	
	);

#else

	p = int (x);

#endif

	return (p);
}



int	get_prev_pow_2 (uint32_t x)
{
	assert (x > 0);

#if defined (_MSC_VER)

	#if ((_MSC_VER / 100) < 14)

	int				p;
	__asm
	{
		xor				eax, eax
		bsr				eax, x
		mov				p, eax
	}

	#else

	unsigned long	p;
	_BitScanReverse (&p, x);

	#endif

#else

	int				p = -1;

	while ((x & ~(uint32_t (0xFFFF))) != 0)
	{
		p += 16;
		x >>= 16;
	}
	while ((x & ~(uint32_t (0xF))) != 0)
	{
		p += 4;
		x >>= 4;
	}
	while (x > 0)
	{
		++p;
		x >>= 1;
	}

#endif

	return (int (p));
}



double	sinc (double x)
{
	if (x == 0)
	{
		return (1);
	}

	const double	xp = x * PI;

	return (sin (xp) / xp);
}



/*\\\ PROTECTED \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



/*\\\ PRIVATE \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/



#endif	// fnc_CODEHEADER_INCLUDED



/*\\\ EOF \\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
